structure Pnm =
struct

  exception Pnm of string

  fun fromToks ("P3" :: ws :: hs :: cs :: data) = 
      let
	  val w = (case Int.fromString ws of
		      NONE => raise Pnm "bad width"
		    | SOME w => w)
	  val h = (case Int.fromString hs of
		      NONE => raise Pnm "bad height"
		    | SOME h => h)
	  val () = (case Int.fromString cs of
		      NONE => raise Pnm "bad maxcolor"
		    | SOME 255 => ()
		    | SOME h => raise Pnm "unsupported maxcolor")
	  val intdata = map (fn x => Option.valOf (Int.fromString x)) data handle Option => raise Pnm "bad data element"
	  fun cook [] = []
	    | cook (r :: g :: b :: tl) = 
	      let
		  open Word32
	      in
		  (<<(0wxff, 0w24) + <<(fromInt r, 0w16) + <<(fromInt g, 0w8) + fromInt b) :: cook tl
	      end
	    | cook _ = raise Pnm "bad data length"
      in
	  {w = w, h = h, pixels = Array.fromList (cook intdata)}
      end
    | fromToks ["P1", _] = raise Pnm "unsupported Pnm format"
    | fromToks ["P2", _] = raise Pnm "unsupported Pnm format"
    | fromToks ["P4", _] = raise Pnm "unsupported Pnm format"
    | fromToks ["P5", _] = raise Pnm "unsupported Pnm format"
    | fromToks ["P6", _] = raise Pnm "unsupported Pnm format"
    | fromToks _ = raise Pnm "ill-formed pnm file"
      
  fun stripcomments s = 
      let
	  val comment = StringUtil.charspec "#"
	  val newline = StringUtil.charspec "\n\r"
	  fun go (c::tl) true = if comment c 
				then go tl false
				else c :: go tl true
	    | go (c::tl) false = if newline c
				 then go tl true
				 else go tl false 
	    | go [] _ = []
      in
	  implode (go (explode s) true)
      end

  fun fromFile fname = fromToks (String.tokens StringUtil.whitespec
					       (stripcomments (StringUtil.readfile fname)))
end
