
#include <SDL.h>
#include <windows.h>

Uint32 timer_cb(Uint32 interval, void *param) {
  SDL_Event user_event;
  user_event.type=SDL_USEREVENT;
  user_event.user.code=2;
  user_event.user.data1=NULL;
  user_event.user.data2=NULL;
  SDL_PushEvent(&user_event);
  return (Uint32)param;
}

int ml_init() {
  if (SDL_Init (SDL_INIT_VIDEO | 
                SDL_INIT_TIMER | 
		/* for debugging */
		SDL_INIT_NOPARACHUTE |
		SDL_INIT_AUDIO) < 0) {
    
    printf("Unable to initialize SDL. (%s)\n", SDL_GetError());
    return 0;
  }
  else 
    return 1;
}

SDL_TimerID ml_startclock(int timeout) {
  SDL_TimerID my_timer_id = SDL_AddTimer(timeout, timer_cb, (void *)timeout);    
  if (my_timer_id == NULL) {
    printf("Error starting clock.\n");
    return NULL;
  }
  return my_timer_id;
}

void ml_stopclock(SDL_TimerID timer) {
  if (!SDL_RemoveTimer(timer)) {
    printf("Error stopping clock.\n");
  }
}

SDL_Surface * ml_makescreen(int w, int h) {
  SDL_Surface * ret = SDL_SetVideoMode(w, h, 32,
				       SDL_SWSURFACE |
				       SDL_RESIZABLE);
  SDL_WM_SetCaption  ("Tiley Game", 0);

  return ret;
}

void ml_blitall(SDL_Surface * src, SDL_Surface * dst, int x, int y) {
  SDL_Rect r;
  r.x = x;
  r.y = y;
  SDL_BlitSurface(src, 0, dst, &r);
}

void ml_blit(SDL_Surface * src, SDL_Surface * dst, int sx, int sy, int w, int h, int dx, int dy) {
  SDL_Rect sr, dr;
  sr.x = sx;
  sr.y = sy;
  sr.w = w;
  sr.h = h;
  dr.x = dx;
  dr.y = dy;
  SDL_BlitSurface(src, &sr, dst, &dr);
}

int ml_surfacewidth(SDL_Surface * src) {
  return src->w;
}

int ml_surfaceheight(SDL_Surface * src) {
  return src->h;
}

void ml_fillrect(SDL_Surface * s, int x, int y, int w, int h, Uint32 color) {
  SDL_Rect r;
  r.x = x;
  r.y = y;
  r.w = w;
  r.h = h;
  SDL_FillRect(s, &r, color);
}

void ml_clearsurface(SDL_Surface * s, Uint32 color) {
  SDL_FillRect(s, 0, color);
}

SDL_Event * ml_newevent() {
  return (SDL_Event*) malloc (sizeof (SDL_Event));
}

int ml_eventtag(SDL_Event * e) {
  return e->type;
}

int ml_event8_2nd(SDL_Event * e) {
  return (((SDL_KeyboardEvent*)e)->which);
}

int ml_event8_3rd(SDL_Event * e) {
  return (((SDL_KeyboardEvent*)e)->state);
}

int ml_event_keyboard_sym(SDL_KeyboardEvent* e) {
  return e->keysym.sym;
}

int ml_event_keyboard_mod(SDL_KeyboardEvent* e) {
  return e->keysym.mod;
}

int ml_event_keyboard_unicode(SDL_KeyboardEvent* e) {
  return e->keysym.unicode;
}

void maybe_lock(SDL_Surface *s) {
  if ( SDL_MUSTLOCK(s) ) {
    if ( SDL_LockSurface(s) < 0 ) {
      fprintf(stderr, "Can't lock surface: %s\n", SDL_GetError());
      return;
    }
  }
}

void maybe_unlock(SDL_Surface *s) {
  if ( SDL_MUSTLOCK(s) ) {
    SDL_UnlockSurface(s);
  }
}

SDL_Surface *ml_newsurface(int w, int h, char *pixels) {
  SDL_Surface *img = SDL_CreateRGBSurface  (SDL_SWSURFACE, w, h, 32,
					    0x00ff0000L, 
					    0x0000ff00L,
					    0x000000ffL,
					    0xff000000L);

  maybe_lock(img);
 
  {
    int x, y;
    
    for (y = 0; y < h; y++) {
      for (x = 0; x < w; x++) {
	Uint8 *dst = (Uint8 *)img->pixels + y * img->pitch + x * 4;	
	Uint8 *src = (Uint8 *)pixels + y * img->pitch + x * 4;
        *(Uint32 *)dst = *(Uint32 *)src;
      }
    }

  }
   
  maybe_unlock(img);

  SDL_UpdateRect(img, 0, 0, 16, 16);

  return img;
}

SDL_Surface *ml_create_image() {
  SDL_Surface *img = SDL_CreateRGBSurface  (SDL_SWSURFACE, 64, 64, 32,
					    0x00ff0000L, 
					    0x0000ff00L,
					    0x000000ffL,
					    0xff000000L);
  maybe_lock(img);
 
  {
    int x, y;
    
    for (y = 0; y < 64; y++) {
      for (x = 0; x < 64; x++) {
	Uint8 *p = (Uint8 *)img->pixels + y * img->pitch + x * 4;	
        *(Uint32 *)p = 0x00000000L + ((x ^ y) << 24) + (x << 2) + (y << 10);	
      }
    }

  }
   
  maybe_unlock(img);
  SDL_UpdateRect(img, 0, 0, 16, 16);
  return img;
}

/*
 * Set the pixel at (x, y) to the given value
 * NOTE: The surface must be locked before calling this!
 */
void ml_putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
  
  if ( SDL_MUSTLOCK(surface) ) {
    if ( SDL_LockSurface(surface) < 0 ) {
      fprintf(stderr, "Can't lock surface: %s\n", SDL_GetError());
      return;
    }
  }
  
    {
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
    }

    if ( SDL_MUSTLOCK(surface) ) {
        SDL_UnlockSurface(surface);
    }
    /* Update just the part of the display that we've changed */
    SDL_UpdateRect(surface, x, y, 1, 1);

}

int ml_mouse_x() {
  int x;
  SDL_GetMouseState(&x, NULL);
  return x;
}

int ml_mouse_y() {
  int y;
  SDL_GetMouseState(NULL, &y);
  return y;
}

int ml_mouse_buttons() {
  return SDL_GetMouseState(NULL, NULL);
}
