structure RF = RasterFont(structure Sdl = Sdl);
structure GS = GameState(structure Sdl = Sdl);

structure Test =
struct

  structure S = Sdl

  val () = S.init()
  val width = 640
  val height = 480

  val screen = S.mkscreen (width, height)
(*  val img = S.create_image() *)

  fun newsurface_fromppm fname =
      let
	  val {w, h, pixels} = Pnm.fromFile fname
	  val _ = Array.modify (fn x => if x = 0wxff00ffff then 0wx00000000 else x) pixels (* cyan key *)
	  val img = S.newsurface (w, h, pixels)
      in
	  img
      end

  val img = newsurface_fromppm "tiles.ppm"
  val font = RF.newfont {w = 6,
			 h = 12,
			 firstchar = 32,
			 charperrow = 16,
			 pixels = newsurface_fromppm "nitz.ppm"}
         
(*   val () = messagebox "ok?" *)

(*  val w = surface_width graphic
  val h = surface_height graphic *)

  val w = 32
  val h = 32


(*   val _ = S.blitall(img, screen,150,100) *)
  val clockid = S.startclock 250


  fun redraw (l,eff) = (S.clearsurface (screen, 0wx7777777);
		  GS.drawTo(l, eff, img, screen);
		  RF.blitstring(font, screen, w * GS.w + 64, 32, "Score:\n " ^ Int.toString (GS.getScore l));
		  S.flip(screen))


  val initstate = GS.initial [GS.RED, GS.BLUE, GS.GREEN, GS.GRAY,
			      GS.RED, GS.GREEN, GS.BLUE, GS.GRAY,
			      GS.RED, GS.BLUE, GS.BLUE, GS.GREEN,
			      GS.BLUE, GS.GREEN, GS.GRAY, GS.RED,
			      GS.BLUE, GS.RED, GS.GREEN, GS.GRAY,
			      GS.RED, GS.BLUE, GS.GRAY,  GS.GREEN, 
			      GS.RED, GS.GREEN, GS.GRAY, GS.GRAY,
			      GS.RED, GS.GREEN, GS.BLUE, GS.GRAY,
			      GS.GRAY, GS.RED, GS.BLUE, GS.GREEN, 
			      GS.RED, GS.BLUE, GS.GREEN, GS.GRAY]

  fun do_mouse {x,y,s} =
      if s = 1
      then (S.blit(img, screen, 2 * 32, 0, 32, 32, x-16,y-16); S.flip screen)
      else if s = 2
      then S.stopclock clockid
      else if s = 4 
      then (S.blit(img, screen, 1 * 32, 32, 32, 32, x-16,y-16); S.flip screen)
      else ()

  and make_move (l, m) = 
      let val (l,eff) = (case GS.update (l,m) of SOME x => x | NONE => (l,GS.noeffect))
      in  redraw (l,eff); loop (l,eff)
      end

  and loop (l,eff) =
    case S.waitevent () of
      SOME (S.E_KeyDown { sym = S.Key.SDLK_UP }) => make_move (l, GS.N)
    | SOME (S.E_KeyDown { sym = S.Key.SDLK_LEFT }) => make_move (l, GS.W)
    | SOME (S.E_KeyDown { sym = S.Key.SDLK_RIGHT }) => make_move (l, GS.E)
    | SOME (S.E_KeyDown { sym = S.Key.SDLK_DOWN }) => make_move (l, GS.S)
    | SOME (S.E_KeyDown { sym = S.Key.SDLK_q }) => ()
    | SOME (S.E_KeyDown { sym = S.Key.SDLK_RETURN }) => let val l = initstate in redraw (l,GS.noeffect); loop (l,GS.noeffect) end
    | SOME (S.E_KeyDown _) => ((* print "Key"; *) loop (l,eff))
    | SOME (S.E_MouseDown inf) => let val l = initstate in redraw (l,GS.noeffect); loop (l,GS.noeffect) end
(*     | SOME (S.E_MouseMotion inf) => (do_mouse inf; loop l)*)
    | SOME S.E_Quit => ()
    | SOME S.E_User => (GS.drawcursor := not (!GS.drawcursor); redraw (l,eff); loop (l,eff))
    | _ => loop (l,eff)

  val _ = redraw (initstate,GS.noeffect)
  val () = loop (initstate,GS.noeffect)
    handle e => S.messagebox ("Uncaught exception: " ^ exnName e ^ " / " ^ exnMessage e)

end
