signature SDL =
sig
    type surface
    type timer_id
    type color = Word32.word
    
    structure Key : KEY

    datatype event =
	     E_Active
	   | E_KeyDown of { sym : Key.sdlk }
	   | E_KeyUp of { sym : Key.sdlk }
	   | E_MouseMotion of { x : int, y : int, s : int }
	   | E_MouseDown of { x : int, y : int, s : int }
	   | E_MouseUp of { x : int, y : int, s : int }
	   | E_JoyAxis
	   | E_JoyDown
	   | E_JoyUp
	   | E_JoyHat
	   | E_JoyBall
	   | E_Resize
	   | E_Expose
	   | E_SysWM
	   | E_User
	   | E_Quit
	   | E_Unknown

    val init : unit -> unit
    val messagebox : string -> unit
    val putpixel : surface * int * int * color -> unit
    val mkscreen : int * int -> surface
    val flip : surface -> unit
    val blitall : surface * surface * int * int -> unit
    val blit : surface * surface * int * int * int * int * int * int -> unit (* x, y, w, h -> x, y *)
    val clearsurface : surface * color -> unit
    val fillrect : surface * int * int * int * int * color -> unit
    val create_image : unit -> surface
    val newsurface : int * int * color Array.array -> surface
    val startclock : int -> timer_id (* int arg = period in ms *)
    val stopclock : timer_id -> unit
    val waitevent : unit -> event option
end
