structure Sdl :> SDL =
struct

  type ptr = MLton.Pointer.t
  type surface = ptr
  type timer_id = ptr
  type color = Word32.word

  exception Nope

  (* imports *)
  val init_ = _import "ml_init" : unit -> int ;
  val putpixel = _import "ml_putpixel" : ptr * int * int * Word32.word -> unit ;
  val mkscreen = _import "ml_makescreen" : int * int -> ptr ;
  val mb_ = _import "MessageBoxA" stdcall : int * string * string * int -> unit ;
  val flip = _import "SDL_Flip" : ptr -> unit ;
  val blitall = _import "ml_blitall" : ptr * ptr * int * int -> unit ;
  val blit = _import "ml_blit" : ptr * ptr * int * int * int * int * int * int -> unit ;
  val surface_width = _import "ml_surfacewidth" : ptr -> int ;
  val surface_height = _import "ml_surfaceheight" : ptr -> int ;
  val clearsurface = _import "ml_clearsurface" : ptr * Word32.word -> unit ;
  val fillrect = _import "ml_fillrect" : ptr * int * int * int * int * Word32.word -> unit ;
  val newevent = _import "ml_newevent" : unit -> ptr ;
  val free = _import "free" : ptr -> unit ;
  val eventtag = _import "ml_eventtag" : ptr -> int ;
  val event_keyboard_sym = _import "ml_event_keyboard_sym" : ptr -> int ;
  val create_image = _import "ml_create_image" : unit -> ptr ;
  val newsurface = _import "ml_newsurface" : int * int * Word32.word array -> ptr ;
  val mouse_x = _import "ml_mouse_x" : unit -> int ;
  val mouse_y = _import "ml_mouse_y" : unit -> int ;
  val mouse_buttons = _import "ml_mouse_buttons" : unit -> int ;
  val startclock = _import "ml_startclock" : int -> ptr ;
  val stopclock = _import "ml_stopclock" : ptr -> unit ;

  structure Key = Key

  datatype event =
    E_Active
  | E_KeyDown of { sym : Key.sdlk }
  | E_KeyUp of { sym : Key.sdlk }
  | E_MouseMotion of { x : int, y : int, s : int }
  | E_MouseDown of { x : int, y : int, s : int }
  | E_MouseUp of { x : int, y : int, s : int }
  | E_JoyAxis
  | E_JoyDown
  | E_JoyUp
  | E_JoyHat
  | E_JoyBall
  | E_Resize
  | E_Expose
  | E_SysWM
  | E_User
  | E_Quit
  | E_Unknown

  (* more functions *)
  fun messagebox s = mb_ (0, s ^ "\000", "message", 0)

  fun init () = case init_ () of
		    0 => 
		    let in
			messagebox "SDL failed to initialize.";
			raise Nope
		    end
		  | _ => ()
			 
  fun event_mouse_state () = { x = mouse_x(), y = mouse_y(), s = mouse_buttons()}

  fun convertevent e =
   (* XXX need to get props too... *)
    (case eventtag e of
       1 => E_Active
         (* XXX more... *)
     | 2 => E_KeyDown { sym = Key.sdlkey (event_keyboard_sym e) }
     | 3 => E_KeyUp { sym = Key.sdlkey (event_keyboard_sym e) }
     | 4 => E_MouseMotion (event_mouse_state ())
     | 5 => E_MouseDown (event_mouse_state ())
     | 6 => E_MouseUp (event_mouse_state ())
     | 7 => E_JoyAxis
     | 8 => E_JoyBall
     | 9 => E_JoyHat
     | 10 => E_JoyDown
     | 11 => E_JoyUp
     | 12 => E_Quit
     | 13 => E_SysWM
(* reserved..
     | 14 => 
     | 15 =>
*)
     | 16 => E_Resize
     | 17 => E_Expose
     | 24 => E_User (* arises from clock *)
     | _ => E_Unknown
       )

  fun waitevent () =
    let
      val p = _import "SDL_WaitEvent" : ptr -> int ;
      val e = newevent ()
    in
      case p e of
        0 => 
          let in
          (* no event *)
            free e;
            NONE
          end
      | _ =>
          let 
            val ret = convertevent e
          in
            free e;
            SOME ret
          end
    end

end
